"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.fromUint8Array = exports.toUint8Array = exports.bignum = exports.b58 = exports.b64 = exports.hex = exports.isBase64Encoded = exports.stripPrefix = void 0;
const base_1 = require("@scure/base");
const stripPrefix = (prefix, str) => str.startsWith(prefix) ? str.slice(prefix.length) : str;
exports.stripPrefix = stripPrefix;
// This function uses a regex string to check if the input could
// possibly be base64 encoded.
// WARNING:  There are clear text strings that are NOT base64 encoded
//           that will pass this check.
function isBase64Encoded(input) {
    const b64Regex = new RegExp('^(?:[A-Za-z0-9+/]{4})*(?:[A-Za-z0-9+/]{2}==|[A-Za-z0-9+/]{3}=)?$');
    return b64Regex.test(input);
}
exports.isBase64Encoded = isBase64Encoded;
function isHexEncoded(input) {
    const regex = /^[0-9a-fA-F]+$/;
    const normalized = (0, exports.stripPrefix)('0x', input);
    return normalized.length % 2 === 0 && regex.test(normalized);
}
exports.hex = {
    valid: isHexEncoded,
    decode: (input) => base_1.base16.decode((0, exports.stripPrefix)('0x', input).toUpperCase()),
    encode: (input, prefix = false) => {
        input = typeof input === "string" ? (0, exports.toUint8Array)(input) : input;
        return (prefix ? "0x" : "") + base_1.base16.encode(input).toLowerCase();
    }
};
exports.b64 = {
    valid: isBase64Encoded,
    decode: base_1.base64.decode,
    encode: (input) => base_1.base64.encode(typeof input === "string" ? (0, exports.toUint8Array)(input) : input)
};
exports.b58 = {
    decode: base_1.base58.decode,
    encode: (input) => base_1.base58.encode(typeof input === "string" ? (0, exports.toUint8Array)(input) : input),
};
exports.bignum = {
    decode: (input) => BigInt(input),
    encode: (input, prefix = false) => (prefix ? "0x" : "") + input.toString(16)
};
const toUint8Array = (value) => {
    if (typeof value === "bigint")
        return (0, exports.toUint8Array)(exports.bignum.encode(value));
    return (new TextEncoder()).encode(value);
};
exports.toUint8Array = toUint8Array;
const fromUint8Array = (value) => {
    return (new TextDecoder()).decode(value);
};
exports.fromUint8Array = fromUint8Array;
//# sourceMappingURL=encoding.js.map