"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.addFixedValues = exports.dynamicItemsOfLayout = exports.fixedItemsOfLayout = void 0;
const layout_1 = require("./layout");
function filterItem(item, fixed) {
    switch (item.binary) {
        case "uint":
        case "bytes": {
            const isFixedItem = item["custom"] !== undefined && ((0, layout_1.isPrimitiveType)(item["custom"]) || (0, layout_1.isPrimitiveType)(item["custom"].from));
            return (fixed && isFixedItem || !fixed && !isFixedItem) ? item : null;
        }
        case "array": {
            const filtered = filterItem(item.arrayItem, fixed);
            return (filtered !== null) ? { ...item, arrayItem: filtered } : null;
        }
        case "object": {
            const filteredItems = filterItemsOfLayout(item.layout, fixed);
            return (filteredItems.length > 0) ? { ...item, layout: filteredItems } : null;
        }
        case "switch": {
            const filteredIdLayoutPairs = item.idLayoutPairs.reduce((acc, [idOrConversionId, idLayout]) => {
                const filteredItems = filterItemsOfLayout(idLayout, fixed);
                return filteredItems.length > 0
                    ? [...acc, [idOrConversionId, filteredItems]]
                    : acc;
            }, []);
            return { ...item, idLayoutPairs: filteredIdLayoutPairs };
        }
    }
}
function filterItemsOfLayout(layout, fixed) {
    return layout.reduce((acc, item) => {
        const filtered = filterItem(item, fixed);
        return filtered !== null ? [...acc, filtered] : acc;
    }, []);
}
const fixedItemsOfLayout = (layout) => filterItemsOfLayout(layout, true);
exports.fixedItemsOfLayout = fixedItemsOfLayout;
const dynamicItemsOfLayout = (layout) => filterItemsOfLayout(layout, false);
exports.dynamicItemsOfLayout = dynamicItemsOfLayout;
function addFixedValues(layout, dynamicValues) {
    const ret = {};
    for (const item of layout) {
        const fromDynamic = () => dynamicValues[item.name];
        switch (item.binary) {
            case "uint":
            case "bytes": {
                if (!item?.omit)
                    ret[item.name] = (item.custom !== undefined &&
                        ((0, layout_1.isPrimitiveType)(item.custom) || (0, layout_1.isPrimitiveType)(item.custom.from))) ? ((0, layout_1.isPrimitiveType)(item.custom) ? item.custom : item.custom.to)
                        : fromDynamic();
                break;
            }
            case "array": {
                if (item.name in dynamicValues) {
                    ret[item.name] = (item.arrayItem.binary !== "object")
                        ? fromDynamic()
                        : fromDynamic().map(element => addFixedValues(item.arrayItem.layout, element));
                }
                break;
            }
            case "object": {
                const subDynamicValues = ((item.name in dynamicValues) ? fromDynamic() : {});
                ret[item.name] = addFixedValues(item.layout, subDynamicValues);
                break;
            }
            case "switch": {
                const id = fromDynamic()[item.idTag ?? "id"];
                const [_, idLayout] = item.idLayoutPairs.find(([idOrConversionId]) => (Array.isArray(idOrConversionId) ? idOrConversionId[1] : idOrConversionId) == id);
                ret[item.name] = {
                    [item.idTag ?? "id"]: id,
                    ...addFixedValues(idLayout, fromDynamic())
                };
                break;
            }
        }
    }
    return ret;
}
exports.addFixedValues = addFixedValues;
//# sourceMappingURL=fixedDynamic.js.map